"""
* You might think that the event argument is redundant. However, this allows
  one to point multiple events at one handler.

* [status] Unhandled event: ('ERROR', 'ERROR :Closing Link: [127.0.0.1] by wobble. (Your host is trying to (re)connect too fast -- throttled)')
  WTF?

* events:
      The usual IRC numerics:
          ...
      The usual IRC events:
          WHO
          PRIVMSG
          NOTICE
      PyRC-specific events:
          status: normal happy status type messages, such as loading a script
          error: normal unhappy status type messages, such as failing to load
              a script
          debug: a convenience for scripters.
          unknown: called when another event handler does not exist

  dests:
      '.status': status messages, and anything not belonging to any other
          window (second value in tuple currently ignored)
      '#channel': channel
      'nickname': private chat (However, I feel (slightly) tempted to change
          this to nick!user@host

  event_*(event,args):
      Normally should never do output themselves, except via display()
      format of args depends on the event. For standard IRC events args should
          be the string received from the connection unchanged.

  display_*(event,args):
      format of args depends on the event

  calling script.event() and script.display() is encouraged over calling
  event_*() and display_*() directly. I don't know if I'll be changing my mind
  on this later or not.

* documentantion for events should hopefully also mention where the event is
  triggered from: either the IRC server or a particular event in the base
  script

* Change arglists from tuples to lists, so as to enable handlers to modify
  their arguments (eg. to provide highlighting, beep filtering etc.)

* Reminder to self: this script wil have *lowest* priority (which means that
  newly loaded scripts therefore have a higher (highest?) priority)

* Must make up my mind whether to use single or double quotes.

332 RPL_TOPIC      "<channel> :<topic>"

353 RPL_NAMREPLY   "<channel> :[[@|+]<nick> [[@|+]<nick> [...]]]"
366 RPL_ENDOFNAMES "<channel> :End of /NAMES list"
 
372 RPL_MOTD       ":- <text>"
375 RPL_MOTDSTART  ":- <server> Message of the day - "
376 RPL_ENDOFMOTD  ":End of /MOTD command"

"""

#from string import split
from ui import echo                      # for talking to the front end
from script import event,display,alias,load # for talking to the back end (*and ourselves*)
from sys import exit
import ui
import conxn
import cfg
import string

split=string.split

def event_PING(args):
    display('status','Ping? Pong!')
    conxn.send('PONG'+args[4:])

def event_NOTICE(args):
    """ from user to user :m!michael@wobble NOTICE mini-mike :xxxxxxxxxxxxxxxx """
    """ from server to user :wobble NOTICE mini-mike :Highest connection count: 1 (1 clients) """
    """ from user to channel :m!michael@wobble NOTICE #t :foooooooo """
    """ args=':nick!user@host NOTICE #channel|yournick :blah blah blah' """
    """ This may need work, in particular if dest is neither a channel nor your nick """

    event('beep','notice')
    args=split(args,' ',3)
    if args[2][0] in '#&':
        args=(args[0][1:],args[2],args[3][1:]) # source, channel, message
        display('notice_channel',args)  # this should really trigger an *event*.
    else:
        source=split(args[0][1:],'!',1)
        if len(source)>1:                # from a normal user?
            args=(args[0][1:],args[3][1:]) # source, message
            name=split(args[0],'!',1)[0] # remove stuff after ! (if present...)
#            if not ui.find(name):
#                ui.QueryWindow(name,ui.Rect(35,25,95,41)) # this should be in an event handler
            display('notice_private',args)
        else:
            args=(args[0][1:],args[2],args[3][1:]) # source, dest, message
            display('notice_other',args) # this should really trigger an *event*.

def event_PRIVMSG(args):
    """ 
        args=':[nick!user@]host PRIVMSG #channel|yournick :text'
        This may need work, in particular if dest is neither a channel nor your
        nick. It also needs to trigger *events*, not *displays*.

        CTCP handling may be a bit crockish, and seems to indicate that this
        event handler needs more work than I thought.
    """ 

    args=split(args,' ',3)
    if args[3][1]==chr(1): # are we looking at a CTCP?
        ctcp=split(args[3][2:-1],' ',1)
        if len(ctcp)==1: ctcp.append('')
        event('ctcp_'+ctcp[0],(args[0][1:],args[2],ctcp[1])) # src, dest, text
        return

    if args[2][0] in '#&':
        args=(args[0][1:],args[2],args[3][1:]) # source, channel, message
        display('privmsg_channel',args)  # this should really trigger an *event*.
    else:
        source=split(args[0][1:],'!',1)
        if len(source)>1:
            args=(args[0][1:],args[3][1:]) # source, message
            name=split(args[0],'!',1)[0] # remove stuff after ! (if present...)
            if not ui.find(name):
                ui.QueryWindow(name,ui.Rect(20,1,111,23))
            event('beep','private')
            display('privmsg_private',args)
        else:
            args=(args[0][1:],args[2],args[3][1:]) # source, dest, message
            display('privmsg_other',args) # this should really trigger an *event*.

def event_ctcp_ACTION(args):
    """ args=('source','dest','message') """
    if args[1][0] in '#&':
        display('privmsg_channelaction',args)
    else:
        source=split(args[0],'!',1)
        if len(source)>1:
            args=(args[0],args[2]) # source, message
            name=split(args[0],'!',1)[0] # remove stuff after ! (if present...)
            if not ui.find(name):
                ui.QueryWindow(name,ui.Rect(20,1,111,23))
            event('beep','private')
            display('privmsg_privateaction',args)
        else:
            display('privmsg_otheraction',args) # this should really trigger an *event*.

def event_MODE(args):
    """
        args=':nick MODE yournick :mode(s) ...'
        args=':nick!user@host MODE channel mode(s) ...' Note lack of colon before modes. Grumble.
        format for servers setting modes goes here
    """ 
#    display('debug',args)
    arglist=split(args,' ',3)
    arglist=(arglist[0][1:],arglist[2],arglist[3][1:])
    if args[1][0] in '#&':
        display('mode_channel',arglist)
    else:
        display('mode_user',arglist)

def event_NICK(args):
    """ args=':nick[!user@host] NICK :newnick' """
    global mynick
    display('debug',args)

    arglist=split(args,' ',2)
    if arglist[2][0]==':':
        arglist[2]=arglist[2][1:] # remove colon before nickname that may or may not be there
    arglist=(arglist[0][1:],arglist[2])
    if split(arglist[0],'!',1)[0]==mynick:
        mynick=arglist[1]
#        display('debug','nick change detected')
    display('nick',arglist)

def event_JOIN(args):
    """ args=':nick!user@host JOIN :#channel' """
    global mynick

    args=split(args,' ',2)
    args=(args[0][1:],args[2][1:])
    if split(args[0],'!',1)[0]==mynick:
        win=ui.ChannelWindow(args[1],ui.Rect(0,1,111,23))
    display('join',args)

def event_PART(args):
    global mynick

    """ args=':^EMMA^!~Dawson@per1-ch02.vianet.net.au PART #perth' """
    args=split(args,' ',3)               # source, PART, channel, reason (usually missing)
    args=(args[0][1:],args[2])
    if split(args[0],'!',1)[0]==mynick:
        ui.remove(args[1])
    display('part',args)

def event_KICK(args):
    """ args=':Z!zed@dead.oz.org KICK #perth Laiana :(thebar) fat ass!' """
    args=split(args,' ',4)               # source, KICK, channel, who, reason
    args=(args[0][1:],args[2],args[3],args[4][1:])
    display('kick',args)

def event_QUIT(args):
    """ args=':elusion!acr@Jedi-257.nw.com.au QUIT :I seek son, decieve none for' """
    args=split(args,' ',2)
    display('quit',(args[0],args[2]))

def event_001(args):
    """ :wobble 001 mini-mike :Welcome to the Internet Relay Network mini-mike """
    global mynick

    arglist=split(args,' ',3)
    mynick=arglist[2]
    display('motd',arglist[3][1:])
#    display('motd',args)

event_003=event_002=event_001
""" :wobble 002 pyrctest :Your host is wobble, running version u2.10.04.resolv9.nmt.egcs4.tok.pten.tlim4.admin.upper4.whisper3.gipl.modeless7 """
""" :wobble 003 pyrctest :This server was created Fri Oct 16 1998 at 12:26:02 EDT """
""" :wobble 004 pyrctest wobble u2.10.04.resolv9.nmt.egcs4.tok.pten.tlim4.admin.upper4.whisper3.gipl.modeless7 dioswkg biklmnopstv """

event_255=event_251=event_001
""" :wobble 251 pyrctest :There are 1 users and 0 invisible on 1 servers """
""" :wobble 255 pyrctest :I have 1 clients and 0 servers """

""" :wobble 254 m 1 :channels formed """

def event_257(args):
    """ :southern.hobart.oz.org 257 pyrctest ::Reminder - IRC is a live medi """
    arglist=split(args,' ',3)
    display('motd',arglist[3][1:])

def event_list(args):
    """ :wobble 321 mini-mike Channel :Users  Name """
    """ :mpx.sydney.oz.org 322 mini-mike #JamesBond 2 :welcome to jamesbond """
    arglist=split(args,' ',5)[3:]
    display('list',arglist)

event_321=event_322=event_list

def event_332(args):
    """ :host 332 yournick #channel :topic """
    arglist=split(args,' ',4)
    arglist=(arglist[3],arglist[4][1:])
    win=ui.find(arglist[0])
    if win:
        win['title'].set(arglist[0]+' '+arglist[1])
    display('topic',arglist)

def event_333(args):
    """ :southern.hobart.oz.org 333 pyrctest #Perth dylan 930664667 """
    """ Used on ozorg and prolly others """
    args=split(args,' ',6)
    display('topicset',(args[3],args[4],args[5]))

"""
:wobble 324 m #t +tnl 42
:wobble 329 m #t 930763922

:wobble 332 m #t :this is a topic
:wobble 333 m #t n 930763941
:wobble 353 m = #t :m @n 
:wobble 366 m #t :End of /NAMES list.

:southern.hobart.oz.org 352 pyrctest #20-25'sBar krattwe 203.37.227.85 aussie.melbourne.oz.org Bug H :3 Katherine Attwell
0                       1   2        3           4       5             6                       7   8 9  10

257 RPL_ADMINLOC1  "
372 RPL_MOTD       ":- <text>"
375 RPL_MOTDSTART  ":- <server> Message of the day - "
376 RPL_ENDOFMOTD  ":End of /MOTD command"
"""

def event_352(args):
    args=split(args,' ',10)
    args=(args[3],args[7],args[4]+'@'+args[5],args[6],args[8],args[9][1:],args[10])
    display('who',args)

def event_353(args):
    """ RPL_NAMREPLY ':wobble 353 pyrctest = #t :pyrctest x' """
    args=split(args,' ',5)
    display('names',(args[4],args[5][1:]))

def event_366(args):
    """ RPL_ENDOFNAMES ':wobble 366 x * :End of /NAMES list.' """
    pass

def event_372(args):
    """ args=':host 372 yournick :blah blah blah' """
    args=split(args,' ',3)[3][1:]
    display('motd',args)

# event_257=event_375=event_376=event_372

event_375=event_376=event_372
""" :wobble 375 pyrctest :- wobble Message of the Day - """
""" :wobble 376 pyrctest :End of /MOTD command. """

def event_513(args):
    display('status','pING? pONG!')
    conxn.send('PONG '+split(args,' ')[-1])

def event_literal(args):
    """ args='whatever' Just display stuff as-is """
    display('literal',split(args,' ',3)[2]+' '+split(args,' ',3)[3])

#event_311=event_312=event_317=event_318=event_321=event_323=event_literal
#event_401=event_421=event_literal

event_334=event_372

""" :mpx.sydney.oz.org 334 mini-mike :Usage on ircII: ^B/QUOTE LIST ^B^_parameters^_ """
""" :wobble 401 mini-mike $* :No such nick """
""" :wobble 403 mini-mike #* :No such channel """
""" :wobble 438 b c :Nick change too fast. Please wait 25 seconds. """
""" :mpx.sydney.oz.org 513 mini-mike :If you can't connect, type /QUOTE PONG 2150757723954648866 or /PONG 2150757723954648866 """

def event_SOCKETERROR(args):
    """ args='(AWordThatI'veDecidedToChopOff) error message' """
    display('error',split(args,' ',1)[1])
    exit(0)

def event_load(args):
    """ args='scriptname' """
    display('load',args)

def event_key(args):
    """ args=(key,window) """
    args[1]['input'].insert(chr(args[0]))

event_ctl_a=event_key # useful for faking CTCP's
event_ctl_b=event_ctl_v=event_ctl_underscore=event_key

def event_ctl_k(args):
    args[1]['input'].insert(chr(3))

def event_ctl_m(args):
    """ args=(key,window) """
    text=args[1]['input'].text
    if text:
        if text[0]=='/':
            event('command',text[1:])
        else:
            target=args[1].target
            if target:
                event('sendmsg',(target,text))
            else:
                display('error','No target for window')
                
    args[1]['input'].enter()

def event_command(args):                 # raw IRC command
    arglist=split(args,' ',1)
    if len(arglist)==1: arglist.append('')
    if alias(arglist[0],arglist[1]):
        conxn.send(args)

def event_ctl_x(args):
    ui.next()

def event_KEY_BACKSPACE(args):
    args[1]['input'].bckspc()

def event_KEY_LEFT(args):
    args[1]['input'].left()

def event_KEY_RIGHT(args):
    args[1]['input'].right()

def event_KEY_UP(args):
    args[1]['input'].prev()

def event_KEY_HOME(args):
    args[1]['input'].home()

def event_KEY_END(args):
    args[1]['input'].end()

def event_debug(args):
    display('.debug',str(args))

def event_unknown(args):
    """ args=(event,args) """
    display('error','['+args[0]+'] '+str(args[1]))

def event_sendmsg(args):
    """ args=(target,text) """
    target,text=args
    display('sendmsg',args)
    conxn.send('privmsg '+target+' :'+text)

def event_beep(args):
    """ args='reason' """
    display('beep',args)

#-----------------------------------------------------------------------------#

def alias_j(arg):
    conxn.send('join '+arg)

def alias_server(args):
    arglist=split(args,' ')
    if len(arglist)==0 or len(arglist)>2:
        display('error','/connect: wrong number of arguments')
    if len(arglist)==1:
        arglist.append(6667)
    else:
        arglist[2]=eval(arglist[2])
    if alias(arglist[0],arglist[1]):
        conxn.send(args)

    conxn.send('QUIT :changing servers')
    display('status','Connecting to '+arglist[0]+':'+str(arglist[1])+'...')
    conxn.connect(arglist[0],arglist[1])
    display('status','Registering...')
    conxn.send('NICK '+cfg.get('nickname'))
    conxn.send('USER '+cfg.get('login')+' * * :'+cfg.get('realname')) # * * is actually clienthost serverhost

def alias_load(args):
    load(args)

def alias_eval(args):
    try:
        result='eval('+args+')='+str(eval(args))
    except:
        result='Whoops, eval() exceptioned'
    display('status',result)

def alias_exec(args):
    try:
        exec args
        result='exec('+args+') failed to raise an exception'
    except:
        result='Whoops, exec exceptioned'
    display('status',result)

def alias_echo(args):
    # need to catch exceptions here
    target=ui.curr_target()
    if not target:
        target='.status'
    echo(target,args)

#def alias_me(args):

def alias_test(args):
    for bg in range(0,16):
        s=''
        for fg in range(0,16):
            s=s+'\003'+str(fg)+','+str(bg)+'xxx'
        echo('test',s)

def event_sendmsg(args):
    """ args=(target,text) """
    target,text=args
    display('sendmsg',args)
    conxn.send('privmsg '+target+' :'+text)

#-----------------------------------------------------------------------------#

def display_motd(args):
    """ args='blah blah blah' """
    echo('.status','*** '+args)

def display_join(args):
    """ args=('[nick!user@]host','#channel') """
    echo(args[1],'*** JOIN: '+args[0])
    echo('.status','*** '+args[0]+' joins '+args[1])

def display_part(args):
    """ args=('[nick!user@]host','#channel') """
    echo(args[1],'*** PART: '+args[0])
    echo('.status','*** '+args[0]+' parts '+args[1])

def display_quit(args):
    """ args=('nick!user@host','message') """
    echo('.status','*** '+args[0]+' quit IRC ('+args[1]+')')

def display_notice_channel(args):
    """ args=('[nick!user@]host','#channel','text') """
    nick=split(args[0],'!')[0]
    if ui.find(args[1]):
        echo(args[1],'-'+nick+'- '+args[2])
    else:
        echo('.status','-'+nick+':'+args[1]+'- '+args[2])

def display_notice_private(args):
    """ args=('[nick!user@]host','text') """
    nick=split(args[0],'!')[0]
    if ui.find(nick):
        echo(nick,'-'+nick+'- '+args[1])
    else:
        echo('.status','-'+nick+'- '+args[1])

def display_notice_other(args):
    """ args=('[nick!user@]host','dest','text') """
    echo('.status','-'+args[0]+':'+args[1]+'- '+args[2])

def display_privmsg_channel(args):
    """ args=('[nick!user@]host','#channel','text') """
    nick=split(args[0],'!')[0]
    if ui.find(args[1]):
        echo(args[1],'<'+nick+'> '+args[2])
    else:
        echo('.status','<'+nick+':'+args[1]+'> '+args[2])

def display_privmsg_private(args):
    """ args=('[nick!user@]host','text') """
    nick=split(args[0],'!')[0]
    if ui.find(nick):
        echo(nick,'<'+nick+'> '+args[1])
    else:
        echo('.status','<'+nick+'> '+args[1])

def display_privmsg_channelaction(args):
    """ args=('[nick!user@]host','#channel','text') """
    nick=split(args[0],'!')[0]
    if ui.find(args[1]):
        echo(args[1],'* '+nick+' '+args[2])
    else:
        echo('.status','* '+nick+':'+args[1]+' '+args[2])

def display_privmsg_privateaction(args):
    """ args=('[nick!user@]host','text') """
    nick=split(args[0],'!')[0]
    if ui.find(nick):
        echo(nick,'* '+nick+' '+args[1])
    else:
        echo('.status','* '+nick+' '+args[1])

def display_literal(args):
    """ args='stuff' """
    echo('.status','*** '+args)

def display_load(args):
    """ args=('scriptname','actionperformed') """
    echo('.status','--- '+args[0]+' script '+args[1]+'.')
    
def display_debug(args):
    """ args=anything at all """
    echo('.debug',str(args))

def display_status(args):
    """ args=anything at all """
    echo('.status','*** '+str(args))
display_error=display_status

display_unknown=display_debug

def display_sendmsg(args):
    """ args=(target,text) """
    target,text=args
    echo(target,'> '+text)

def display_mode_user(args):
    """ args=('nick[!user@host]|host','nick','mode(s)') """
    who,nick,modes=args
    echo('.status','*** '+who+' sets mode '+nick+' '+modes)

def display_mode_channel(args):
    """ args=('nick[!user@host]|host','#channel','mode(s)') """
    who,channel,modes=args
    echo(channel,'*** '+who+' sets mode '+modes)

def display_nick(args):
    """ args=('nick[!user@host]','newnick') """
    nick,newnick=args
    echo('.status','*** '+nick+' is now known as '+newnick)

def display_topic(args):
    """ args=('#channel','topic') """
    channel,topic=args
    echo(channel,'*** Topic: '+topic)

def display_topicset(args):
    """ ('#channel','who','when') """
    channel,who,when=args
    echo(channel,'*** Topic set by '+who+' at, er, '+when)

def display_beep(args):
    """ args='reason' """
    echo('.status','*beep!* '+str(args))

def display_kick(args):
    """ args=('kicker','#channel','kickee','reason') """
    echo(args[1],'*** '+args[2]+' was kicked by '+args[0]+' ('+args[3]+')')

def display_names(args):
    """ args=('#channel','nicks') """
    channel,nicks=args
    if ui.find(channel):
        echo(channel,'*** Users: '+nicks)
    else:
        echo('.status','*** Users on '+channel+': '+nicks)

def display_who(args):
    """ args=('#channel','nickname','userhost','server','status','hops','fullname' """
    channel,nick,userhost,server,status,hops,name=args
    if ui.find(channel):
        echo(channel,'*** '+nick+' ('+userhost+') '+status+' '+split(server,'.',1)[0]+' '+name)
    else:
        echo('.status','*** '+channel+': '+nick+' ('+userhost+') '+status+' '+split(server,'.',1)[0]+' '+name)

def display_list(args):
    """ args=('channel','users','topic') """
    channel,users,topic=args
    echo('.status','*** '+string.ljust(channel,15)+' '+string.rjust(users,7)+' '+topic)